from datetime import date
import pendulum
import singer
from singer import bookmarks as bks_

from .utils import trim_members_all, clean_for_cache

class Context(object):
    """Represents a collection of global objects necessary for performing
    discovery or for running syncs. Notably, it contains
    - config  - The JSON structure from the config.json argument
    - state   - The mutable state dict that is shared among streams
    - endpoint  - The SendGrid endpoint
    - catalog - A singer.catalog.Catalog. Note this will be None during
                discovery.
    - cache   - A place for streams to store data so it can be shared between
                streams.
    """

    def __init__(self, config, state):
        self.config = config
        self.state = state
        self._catalog = None
        self.selected_stream_ids = None
        self.selected_catalog = None
        self.cache = {}
        self.now = pendulum.now()
        self.now_seconds = pendulum.now().int_timestamp

    @property
    def catalog(self):
        return self._catalog

    @catalog.setter
    def catalog(self, catalog):
        self._catalog = catalog
        self.selected_stream_ids = set(
            [s.tap_stream_id for s in catalog.streams
             if s.is_selected()]
        )
        self.selected_catalog = [s for s in catalog.streams
                                 if s.is_selected()]

    def get_bookmark(self, path):
        return bks_.get_bookmark(self.state, *path)

    def set_bookmark(self, path, val):
        if isinstance(val, date):
            val = val.isoformat()
        bks_.write_bookmark(self.state, path[0], path[1], val)
        self.write_state()

    def get_offset(self, path):
        off = bks_.get_offset(self.state, path[0])
        return (off or {}).get(path[1])

    def set_offset(self, path, val):
        bks_.set_offset(self.state, path[0], path[1], val)

    def clear_offsets(self, tap_stream_id):
        bks_.clear_offset(self.state, tap_stream_id)

    def update_start_date_bookmark(self, path):
        val = self.get_bookmark(path)
        if path[1] == 'member_count':
            if not val:
                val = []
                self.set_bookmark(path, val)
            return val
        else:
            if not val:
                val = self.config["start_date"]
                self.set_bookmark(path, val)
            return pendulum.parse(val)

    def save_member_count_state(self, s, stream):
        old_state = self.update_start_date_bookmark(stream.bookmark)
        new_state = [s] + [os for os in old_state if os['id'] != s['id']]

        self.set_bookmark(stream.bookmark, new_state)

    def write_state(self):
        singer.write_state(self.state)

    def now_date_str(self):
        return self.now.to_rfc3339_string()

    @staticmethod
    def ts_to_dt(ts):
        return pendulum.from_timestamp(ts).to_rfc3339_string()

    def update_cache(self, data, stream_id):
        self.cache.update({
            trim_members_all(stream_id):
                clean_for_cache(data, stream_id)
        })



import os
import json
from collections import namedtuple
import singer


class Scopes(object):
    source = 'auth_check'
    endpoint = 'https://api.sendgrid.com/v3/scopes'
    scopes = [
        'suppression.read',
        'asm.groups.read',
        'marketing_campaigns.read',
        'templates.read',
        'templates.versions.read'
    ]


class IDS(object):
    GLOBAL_SUPPRESSIONS = "global_suppressions"
    GROUPS_ALL = "groups_all"
    GROUPS_MEMBERS = "groups_members"
    CONTACTS = "contacts"
    LISTS_ALL = "lists_all"
    LISTS_MEMBERS = "lists_members"
    SEGMENTS_ALL = "segments_all"
    SEGMENTS_MEMBERS = "segments_members"
    TEMPLATES_ALL = "templates_all"
    INVALIDS = "invalids"
    BOUNCES = "bounces"
    BLOCKS = "blocks"
    SPAM_REPORTS = "spam_reports"
    CAMPAIGNS = "campaigns"


stream_ids = [getattr(IDS, x) for x in dir(IDS)]

PK_FIELDS = {
    IDS.GLOBAL_SUPPRESSIONS: ["email"],
    IDS.GROUPS_ALL: ["id"],
    IDS.GROUPS_MEMBERS: ["email"],
    IDS.CONTACTS: ["id"],
    IDS.LISTS_ALL: ["id"],
    IDS.LISTS_MEMBERS: ["id"],
    IDS.SEGMENTS_ALL: ["id"],
    IDS.SEGMENTS_MEMBERS: ["id"],
    IDS.TEMPLATES_ALL: ["id"],
    IDS.INVALIDS: ["email"],
    IDS.BOUNCES: ["email"],
    IDS.BLOCKS: ["email"],
    IDS.SPAM_REPORTS: ["email"],
    IDS.CAMPAIGNS: ["id"],
}


class BOOKMARKS(object):
    GLOBAL_SUPPRESSIONS = [IDS.GLOBAL_SUPPRESSIONS, "end_time"]
    GROUPS_MEMBERS = [IDS.GROUPS_MEMBERS, "member_count"]
    CONTACTS = [IDS.CONTACTS, "timestamp"]
    LISTS_MEMBERS = [IDS.LISTS_MEMBERS, "member_count"]
    SEGMENTS_MEMBERS = [IDS.SEGMENTS_MEMBERS, "member_count"]
    INVALIDS = [IDS.INVALIDS, "end_time"]
    BOUNCES = [IDS.BOUNCES, "end_time"]
    BLOCKS = [IDS.BLOCKS, "end_time"]
    SPAM_REPORTS = [IDS.SPAM_REPORTS, "end_time"]


Stream = namedtuple("Stream", ("tap_stream_id", "bookmark", "endpoint"))
STREAMS = [
    Stream(IDS.GLOBAL_SUPPRESSIONS, BOOKMARKS.GLOBAL_SUPPRESSIONS, 'https://api.sendgrid.com/v3/suppression/unsubscribes'),
    Stream(IDS.GROUPS_ALL, None, 'https://api.sendgrid.com/v3/asm/groups'),
    Stream(IDS.GROUPS_MEMBERS, BOOKMARKS.GROUPS_MEMBERS, 'https://api.sendgrid.com/v3/asm/groups/{}/suppressions'),
    Stream(IDS.CONTACTS, BOOKMARKS.CONTACTS, 'https://api.sendgrid.com/v3/contactdb/recipients/search'),
    Stream(IDS.LISTS_ALL, None, 'https://api.sendgrid.com/v3/contactdb/lists'),
    Stream(IDS.LISTS_MEMBERS, BOOKMARKS.LISTS_MEMBERS, 'https://api.sendgrid.com/v3/contactdb/lists/{}/recipients'),
    Stream(IDS.SEGMENTS_ALL, None, 'https://api.sendgrid.com/v3/contactdb/segments'),
    Stream(IDS.SEGMENTS_MEMBERS, BOOKMARKS.SEGMENTS_MEMBERS, 'https://api.sendgrid.com/v3/contactdb/segments/{}/recipients'),
    Stream(IDS.TEMPLATES_ALL, None, 'https://api.sendgrid.com/v3/templates'),
    Stream(IDS.INVALIDS, BOOKMARKS.INVALIDS, 'https://api.sendgrid.com/v3/suppression/invalid_emails'),
    Stream(IDS.BOUNCES, BOOKMARKS.BOUNCES, 'https://api.sendgrid.com/v3/suppression/bounces'),
    Stream(IDS.BLOCKS, BOOKMARKS.BLOCKS, 'https://api.sendgrid.com/v3/suppression/blocks'),
    Stream(IDS.SPAM_REPORTS, BOOKMARKS.SPAM_REPORTS, 'https://api.sendgrid.com/v3/suppression/spam_reports'),
    Stream(IDS.CAMPAIGNS, None, 'https://api.sendgrid.com/v3/campaigns'),
]


def get_abs_path(path):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), path)


def load_schema(stream_id):
    path = 'schemas/{}.json'
    return json.load(open(get_abs_path(path.format(stream_id))))


def load_and_write_schema(tap_stream_id):
    schema = load_schema(tap_stream_id)
    singer.write_schema(tap_stream_id, schema, PK_FIELDS[tap_stream_id])


def write_schema(tap_stream_id, schema):
    singer.write_schema(tap_stream_id, schema.to_dict(), PK_FIELDS[tap_stream_id])


#!/usr/bin/env python

from setuptools import setup

setup(name='tap-sendgrid',
      version='1.0.1',
      description='Singer.io tap for extracting data from the SendGrid API',
      author='Stitch',
      url='http://singer.io',
      classifiers=['Programming Language :: Python :: 3 :: Only'],
      py_modules=['tap_sendgrid'],
      install_requires=['singer-python==5.0.4',
                        'requests==2.20.0',
                        'pendulum==1.2.0',
                        ],
      entry_points='''
          [console_scripts]
          tap-sendgrid=tap_sendgrid:main
      ''',
      packages=['tap_sendgrid'],
      package_data={
          'tap_sendgrid/schemas': [
                "contacts.json",
                "global_suppressions.json",
                "groups_members.json",
                "groups_all.json",
                "invalids.json",
                "lists_all.json",
                "lists_members.json",
                "segments_all.json",
                "segments_members.json",
                "templates_all.json",
                "blocks.json",
                "bounces.json",
                "campaigns.json",
                "spam_reports.json",
              ]
         },
      include_package_data=True
)
